@@ -9,16 +9,15 @@
 /*global _malloc, _free, _memcpy*/
 /*global FUNCTION_TABLE, HEAP8, HEAPU8, HEAP16, HEAPU16, HEAP32, HEAPU32, HEAPF32, HEAPF64*/
 /*global readLatin1String*/
-/*global __emval_register, emval_handle_array, __emval_decref*/
+/*global Emval, emval_handle_array, __emval_decref*/
 /*global ___getTypeName*/
-/*global requireHandle*/
 /*jslint sub:true*/ /* The symbols 'fromWireType' and 'toWireType' must be accessed via array notation to be closure-safe since craftInvokerFunction crafts functions as strings that can't be closured. */
 
 // -- jshint doesn't understand library syntax, so we need to specifically tell it about the symbols we define
 /*global typeDependencies, flushPendingDeletes, getTypeName, getBasestPointer, throwBindingError, UnboundTypeError, _embind_repr, registeredInstances, registeredTypes, getShiftFromSize*/
 /*global ensureOverloadTable, embind__requireFunction, awaitingDependencies, makeLegalFunctionName, embind_charCodes:true, registerType, createNamedFunction, RegisteredPointer, throwInternalError*/
 /*global simpleReadValueFromPointer, floatReadValueFromPointer, integerReadValueFromPointer, enumReadValueFromPointer, replacePublicSymbol, craftInvokerFunction, tupleRegistrations*/
-/*global finalizationGroup, attachFinalizer, detachFinalizer, releaseClassHandle, runDestructor*/
+/*global finalizationRegistry, attachFinalizer, detachFinalizer, releaseClassHandle, runDestructor*/
 /*global ClassHandle, makeClassHandle, structRegistrations, whenDependentTypesAreResolved, BindingError, deletionQueue, delayFunction:true, upcastPointer*/
 /*global exposePublicSymbol, heap32VectorToArray, new_, RegisteredPointer_getPointee, RegisteredPointer_destructor, RegisteredPointer_deleteObject, char_0, char_9*/
 /*global getInheritedInstanceCount, getLiveInheritedInstances, setDelayFunction, InternalError, runDestructors*/
@@ -51,6 +50,9 @@ var LibraryEmbind = {
     Module['flushPendingDeletes'] = flushPendingDeletes;
     Module['setDelayFunction'] = setDelayFunction;
 #if IN_TEST_HARNESS
+#if ASSERTIONS
+    Module['ASSERTIONS'] = true;
+#endif
 #if DYNAMIC_EXECUTION
     // Without dynamic execution, dynamically created functions will have no
     // names. This lets the test suite know that.
@@ -301,9 +303,7 @@ var LibraryEmbind = {
     '$typeDependencies', '$throwBindingError',
     '$whenDependentTypesAreResolved'],
   $registerType__docs: '/** @param {Object=} options */',
-  $registerType: function(rawType, registeredInstance, options) {
-    options = options || {};
-
+  $registerType: function(rawType, registeredInstance, options = {}) {
     if (!('argPackAdvance' in registeredInstance)) {
         throw new TypeError('registerType registeredInstance requires argPackAdvance');
     }
@@ -405,6 +405,7 @@ var LibraryEmbind = {
   },
 
   $heap32VectorToArray: function(count, firstElement) {
+    {{{ from64('firstElement') }}}
     var array = [];
     for (var i = 0; i < count; i++) {
         array.push(HEAP32[(firstElement >> 2) + i]);
@@ -476,6 +477,7 @@ var LibraryEmbind = {
 
   $getShiftFromSize__deps: [],
   $getShiftFromSize: function(size) {
+    {{{ from64('size') }}}
     switch (size) {
         case 1: return 0;
         case 2: return 1;
@@ -499,6 +501,11 @@ var LibraryEmbind = {
         case 2: return signed ?
             function readS32FromPointer(pointer) { return HEAP32[pointer >> 2]; } :
             function readU32FromPointer(pointer) { return HEAPU32[pointer >> 2]; };
+#if WASM_BIGINT
+        case 3: return signed ?
+            function readS64FromPointer(pointer) { return HEAP64[pointer >> 3]; } :
+            function readU64FromPointer(pointer) { return HEAPU64[pointer >> 3]; };
+#endif
         default:
             throw new TypeError("Unknown integer type: " + name);
     }
@@ -551,39 +558,87 @@ var LibraryEmbind = {
 
     var shift = getShiftFromSize(size);
 
-    var fromWireType = function(value) {
-        return value;
-    };
+    var fromWireType = (value) => value;
 
     if (minRange === 0) {
         var bitshift = 32 - 8*size;
-        fromWireType = function(value) {
-            return (value << bitshift) >>> bitshift;
-        };
+        fromWireType = (value) => (value << bitshift) >>> bitshift;
     }
 
-    var isUnsignedType = (name.indexOf('unsigned') != -1);
-
+    var isUnsignedType = (name.includes('unsigned'));
+    var checkAssertions = (value, toTypeName) => {
+#if ASSERTIONS
+        if (typeof value != "number" && typeof value != "boolean") {
+            throw new TypeError('Cannot convert "' + _embind_repr(value) + '" to ' + toTypeName);
+        }
+        if (value < minRange || value > maxRange) {
+            throw new TypeError('Passing a number "' + _embind_repr(value) + '" from JS side to C/C++ side to an argument of type "' + name + '", which is outside the valid range [' + minRange + ', ' + maxRange + ']!');
+        }
+#endif
+    }
+    var toWireType;
+    if (isUnsignedType) {
+        toWireType = function(destructors, value) {
+            checkAssertions(value, this.name);
+            return value >>> 0;
+        }
+    } else {
+        toWireType = function(destructors, value) {
+            checkAssertions(value, this.name);
+            // The VM will perform JS to Wasm value conversion, according to the spec:
+            // https://www.w3.org/TR/wasm-js-api-1/#towebassemblyvalue
+            return value;
+        }
+    }
     registerType(primitiveType, {
         name: name,
         'fromWireType': fromWireType,
-        'toWireType': function(destructors, value) {
-            // todo: Here we have an opportunity for -O3 level "unsafe" optimizations: we could
-            // avoid the following two if()s and assume value is of proper type.
-            if (typeof value !== "number" && typeof value !== "boolean") {
-                throw new TypeError('Cannot convert "' + _embind_repr(value) + '" to ' + this.name);
-            }
-            if (value < minRange || value > maxRange) {
-                throw new TypeError('Passing a number "' + _embind_repr(value) + '" from JS side to C/C++ side to an argument of type "' + name + '", which is outside the valid range [' + minRange + ', ' + maxRange + ']!');
-            }
-            return isUnsignedType ? (value >>> 0) : (value | 0);
-        },
+        'toWireType': toWireType,
         'argPackAdvance': 8,
         'readValueFromPointer': integerReadValueFromPointer(name, shift, minRange !== 0),
         destructorFunction: null, // This type does not need a destructor
     });
   },
 
+#if WASM_BIGINT
+  _embind_register_bigint__deps: [
+    'embind_repr', '$readLatin1String', '$registerType', '$integerReadValueFromPointer'],
+  _embind_register_bigint: function(primitiveType, name, size, minRange, maxRange) {
+    name = readLatin1String(name);
+
+    var shift = getShiftFromSize(size);
+
+    var isUnsignedType = (name.indexOf('u') != -1);
+
+    // maxRange comes through as -1 for uint64_t (see issue 13902). Work around that temporarily
+    if (isUnsignedType) {
+        // Use string because acorn does recognize bigint literals
+        maxRange = (BigInt(1) << BigInt(64)) - BigInt(1);
+    }
+
+    registerType(primitiveType, {
+        name: name,
+        'fromWireType': function (value) {
+            return value;
+        },
+        'toWireType': function (destructors, value) {
+            if (typeof value != "bigint") {
+                throw new TypeError('Cannot convert "' + _embind_repr(value) + '" to ' + this.name);
+            }
+            if (value < minRange || value > maxRange) {
+                throw new TypeError('Passing a number "' + _embind_repr(value) + '" from JS side to C/C++ side to an argument of type "' + name + '", which is outside the valid range [' + minRange + ', ' + maxRange + ']!');
+            }
+            return value;
+        },
+        'argPackAdvance': 8,
+        'readValueFromPointer': integerReadValueFromPointer(name, shift, !isUnsignedType),
+        destructorFunction: null, // This type does not need a destructor
+    });
+  },
+#else
+  _embind_register_bigint__deps: [],
+  _embind_register_bigint: function(primitiveType, name, size, minRange, maxRange) {},
+#endif
 
   _embind_register_float__deps: [
     'embind_repr', '$floatReadValueFromPointer', '$getShiftFromSize',
@@ -594,14 +649,16 @@ var LibraryEmbind = {
     registerType(rawType, {
         name: name,
         'fromWireType': function(value) {
-            return value;
+             return value;
         },
         'toWireType': function(destructors, value) {
-            // todo: Here we have an opportunity for -O3 level "unsafe" optimizations: we could
-            // avoid the following if() and assume value is of proper type.
-            if (typeof value !== "number" && typeof value !== "boolean") {
+#if ASSERTIONS
+            if (typeof value != "number" && typeof value != "boolean") {
                 throw new TypeError('Cannot convert "' + _embind_repr(value) + '" to ' + this.name);
             }
+#endif
+            // The VM will perform JS to Wasm value conversion, according to the spec:
+            // https://www.w3.org/TR/wasm-js-api-1/#towebassemblyvalue
             return value;
         },
         'argPackAdvance': 8,
@@ -669,15 +726,15 @@ var LibraryEmbind = {
             }
 
             var getLength;
-            var valueIsOfTypeString = (typeof value === 'string');
+            var valueIsOfTypeString = (typeof value == 'string');
 
             if (!(valueIsOfTypeString || value instanceof Uint8Array || value instanceof Uint8ClampedArray || value instanceof Int8Array)) {
                 throwBindingError('Cannot pass non-string to std::string');
             }
             if (stdStringIsUTF8 && valueIsOfTypeString) {
-                getLength = function() {return lengthBytesUTF8(value);};
+                getLength = () => lengthBytesUTF8(value);
             } else {
-                getLength = function() {return value.length;};
+                getLength = () => value.length;
             }
 
             // assumes 4-byte alignment
@@ -718,8 +775,12 @@ var LibraryEmbind = {
   },
 
   _embind_register_std_wstring__deps: [
-    '$readLatin1String', '$registerType',
-    '$simpleReadValueFromPointer'],
+    '$readLatin1String', '$registerType', '$simpleReadValueFromPointer',
+#if MINIMAL_RUNTIME
+    '$UTF16ToString', '$stringToUTF16', '$lengthBytesUTF16',
+    '$UTF32ToString', '$stringToUTF32', '$lengthBytesUTF32',
+#endif
+    ],
   _embind_register_std_wstring: function(rawType, charSize, name) {
     name = readLatin1String(name);
     var decodeString, encodeString, getHeap, lengthBytesUTF, shift;
@@ -727,13 +788,13 @@ var LibraryEmbind = {
         decodeString = UTF16ToString;
         encodeString = stringToUTF16;
         lengthBytesUTF = lengthBytesUTF16;
-        getHeap = function() { return HEAPU16; };
+        getHeap = () => HEAPU16;
         shift = 1;
     } else if (charSize === 4) {
         decodeString = UTF32ToString;
         encodeString = stringToUTF32;
         lengthBytesUTF = lengthBytesUTF32;
-        getHeap = function() { return HEAPU32; };
+        getHeap = () => HEAPU32;
         shift = 2;
     }
     registerType(rawType, {
@@ -766,7 +827,7 @@ var LibraryEmbind = {
             return str;
         },
         'toWireType': function(destructors, value) {
-            if (!(typeof value === 'string')) {
+            if (!(typeof value == 'string')) {
                 throwBindingError('Cannot pass non-string to C++ string type ' + name);
             }
 
@@ -792,19 +853,19 @@ var LibraryEmbind = {
   },
 
   _embind_register_emval__deps: [
-    '_emval_decref', '$emval_handle_array', '_emval_register',
+    '_emval_decref', '$Emval',
     '$readLatin1String', '$registerType', '$simpleReadValueFromPointer'],
   _embind_register_emval: function(rawType, name) {
     name = readLatin1String(name);
     registerType(rawType, {
         name: name,
         'fromWireType': function(handle) {
-            var rv = emval_handle_array[handle].value;
+            var rv = Emval.toValue(handle);
             __emval_decref(handle);
             return rv;
         },
         'toWireType': function(destructors, value) {
-            return __emval_register(value);
+            return Emval.toHandle(value);
         },
         'argPackAdvance': 8,
         'readValueFromPointer': simpleReadValueFromPointer,
@@ -826,6 +887,10 @@ var LibraryEmbind = {
         Uint32Array,
         Float32Array,
         Float64Array,
+#if WASM_BIGINT
+        BigInt64Array,
+        BigUint64Array,
+#endif
     ];
 
     var TA = typeMapping[dataTypeIndex];
@@ -894,7 +959,11 @@ var LibraryEmbind = {
   // (hand-written JS code) -> (autogenerated JS invoker) -> (template-generated C++ invoker) -> (target C++ function)
   // craftInvokerFunction generates the JS invoker function for each function exposed to JS through embind.
   $craftInvokerFunction__deps: [
-    '$makeLegalFunctionName', '$new_', '$runDestructors', '$throwBindingError'],
+    '$makeLegalFunctionName', '$new_', '$runDestructors', '$throwBindingError',
+  #if ASYNCIFY
+    '$Asyncify',
+  #endif
+  ],
   $craftInvokerFunction: function(humanName, argTypes, classType, cppInvokerFunc, cppTargetFunc) {
     // humanName: a human-readable string name for the function to be generated.
     // argTypes: An array that contains the embind type objects for all types in the function signature.
@@ -923,7 +992,7 @@ var LibraryEmbind = {
     // TODO: Remove this completely once all function invokers are being dynamically generated.
     var needsDestructorStack = false;
 
-    for(var i = 1; i < argTypes.length; ++i) { // Skip return value at index 0 - it's not deleted here.
+    for (var i = 1; i < argTypes.length; ++i) { // Skip return value at index 0 - it's not deleted here.
         if (argTypes[i] !== null && argTypes[i].destructorFunction === undefined) { // The type does not define a destructor function - must use dynamic stack
             needsDestructorStack = true;
             break;
@@ -951,39 +1020,49 @@ var LibraryEmbind = {
       invokerFuncArgs.length = isClassMethodFunc ? 2 : 1;
       invokerFuncArgs[0] = cppTargetFunc;
       if (isClassMethodFunc) {
-        thisWired = argTypes[1].toWireType(destructors, this);
+        thisWired = argTypes[1]['toWireType'](destructors, this);
         invokerFuncArgs[1] = thisWired;
       }
       for (var i = 0; i < expectedArgCount; ++i) {
-        argsWired[i] = argTypes[i + 2].toWireType(destructors, arguments[i]);
+        argsWired[i] = argTypes[i + 2]['toWireType'](destructors, arguments[i]);
         invokerFuncArgs.push(argsWired[i]);
       }
 
       var rv = cppInvokerFunc.apply(null, invokerFuncArgs);
 
-      if (needsDestructorStack) {
-        runDestructors(destructors);
-      } else {
-        for (var i = isClassMethodFunc ? 1 : 2; i < argTypes.length; i++) {
-          var param = i === 1 ? thisWired : argsWired[i - 2];
-          if (argTypes[i].destructorFunction !== null) {
-            argTypes[i].destructorFunction(param);
+      function onDone(rv) {
+        if (needsDestructorStack) {
+          runDestructors(destructors);
+        } else {
+          for (var i = isClassMethodFunc ? 1 : 2; i < argTypes.length; i++) {
+            var param = i === 1 ? thisWired : argsWired[i - 2];
+            if (argTypes[i].destructorFunction !== null) {
+              argTypes[i].destructorFunction(param);
+            }
           }
         }
+
+  #if EMSCRIPTEN_TRACING
+        Module.emscripten_trace_exit_context();
+  #endif
+
+        if (returns) {
+          return argTypes[0]['fromWireType'](rv);
+        }
       }
 
-#if EMSCRIPTEN_TRACING
-      Module.emscripten_trace_exit_context();
+#if ASYNCIFY
+      if (Asyncify.currData) {
+        return Asyncify.whenDone().then(onDone);
+      }
 #endif
 
-      if (returns) {
-        return argTypes[0].fromWireType(rv);
-      }
+      return onDone(rv);
     };
 #else
     var argsList = "";
     var argsListWired = "";
-    for(var i = 0; i < argCount - 2; ++i) {
+    for (var i = 0; i < argCount - 2; ++i) {
         argsList += (i!==0?", ":"")+"arg"+i;
         argsListWired += (i!==0?", ":"")+"arg"+i+"Wired";
     }
@@ -1016,7 +1095,7 @@ var LibraryEmbind = {
         invokerFnBody += "var thisWired = classParam.toWireType("+dtorStack+", this);\n";
     }
 
-    for(var i = 0; i < argCount - 2; ++i) {
+    for (var i = 0; i < argCount - 2; ++i) {
         invokerFnBody += "var arg"+i+"Wired = argType"+i+".toWireType("+dtorStack+", arg"+i+"); // "+argTypes[i+2].name+"\n";
         args1.push("argType"+i);
         args2.push(argTypes[i+2]);
@@ -1029,10 +1108,16 @@ var LibraryEmbind = {
     invokerFnBody +=
         (returns?"var rv = ":"") + "invoker(fn"+(argsListWired.length>0?", ":"")+argsListWired+");\n";
 
+#if ASYNCIFY
+    args1.push("Asyncify");
+    args2.push(Asyncify);
+    invokerFnBody += "function onDone(" + (returns ? "rv" : "") + ") {\n";
+#endif
+
     if (needsDestructorStack) {
         invokerFnBody += "runDestructors(destructors);\n";
     } else {
-        for(var i = isClassMethodFunc?1:2; i < argTypes.length; ++i) { // Skip return value at index 0 - it's not deleted here. Also skip class type if not a method.
+        for (var i = isClassMethodFunc?1:2; i < argTypes.length; ++i) { // Skip return value at index 0 - it's not deleted here. Also skip class type if not a method.
             var paramName = (i === 1 ? "thisWired" : ("arg"+(i - 2)+"Wired"));
             if (argTypes[i].destructorFunction !== null) {
                 invokerFnBody += paramName+"_dtor("+paramName+"); // "+argTypes[i].name+"\n";
@@ -1053,6 +1138,12 @@ var LibraryEmbind = {
         invokerFnBody += "Module.emscripten_trace_exit_context();\n";
 #endif
     }
+
+#if ASYNCIFY
+    invokerFnBody += "}\n";
+    invokerFnBody += "return Asyncify.currData ? Asyncify.whenDone().then(onDone) : onDone(" + (returns ? "rv" : "") +");\n"
+#endif
+
     invokerFnBody += "}\n";
 
     args1.push(invokerFnBody);
@@ -1075,16 +1166,16 @@ var LibraryEmbind = {
       return getDynCaller(signature, rawFunction);
 #else
 #if !WASM_BIGINT
-      if (signature.indexOf('j') != -1) {
+      if (signature.includes('j')) {
         return getDynCaller(signature, rawFunction);
       }
 #endif
-      return wasmTable.get(rawFunction);
+      return getWasmTableEntry(rawFunction);
 #endif
     }
 
     var fp = makeDynCaller();
-    if (typeof fp !== "function") {
+    if (typeof fp != "function") {
         throwBindingError("unknown function pointer with signature " + signature + ": " + rawFunction);
     }
     return fp;
@@ -1176,10 +1267,10 @@ var LibraryEmbind = {
             var setterArgumentType = elementTypes[i + elementsLength];
             var setter = elt.setter;
             var setterContext = elt.setterContext;
-            elt.read = function(ptr) {
+            elt.read = (ptr) => {
                 return getterReturnType['fromWireType'](getter(getterContext, ptr));
             };
-            elt.write = function(ptr, o) {
+            elt.write = (ptr, o) => {
                 var destructors = [];
                 setter(setterContext, ptr, setterArgumentType['toWireType'](destructors, o));
                 runDestructors(destructors);
@@ -1218,6 +1309,7 @@ var LibraryEmbind = {
 
   $structRegistrations: {},
 
+  _embind_register_value_object__sig: 'viiiiii',
   _embind_register_value_object__deps: [
     '$structRegistrations', '$readLatin1String', '$embind__requireFunction'],
   _embind_register_value_object: function(
@@ -1236,6 +1328,7 @@ var LibraryEmbind = {
     };
   },
 
+  _embind_register_value_object_field__sig: 'viiiiiiiiii',
   _embind_register_value_object_field__deps: [
     '$structRegistrations', '$readLatin1String', '$embind__requireFunction'],
   _embind_register_value_object_field: function(
@@ -1261,6 +1354,7 @@ var LibraryEmbind = {
     });
   },
 
+  _embind_finalize_value_object__sig: 'ii',
   _embind_finalize_value_object__deps: [
     '$structRegistrations', '$runDestructors',
     '$simpleReadValueFromPointer', '$whenDependentTypesAreResolved'],
@@ -1390,7 +1484,7 @@ var LibraryEmbind = {
                     var clonedHandle = handle['clone']();
                     ptr = this.rawShare(
                         ptr,
-                        __emval_register(function() {
+                        Emval.toHandle(function() {
                             clonedHandle['delete']();
                         })
                     );
@@ -1640,39 +1734,53 @@ var LibraryEmbind = {
     }
   },
 
-  $finalizationGroup: false,
+  $finalizationRegistry: false,
 
-  $detachFinalizer_deps: ['$finalizationGroup'],
+  $detachFinalizer_deps: ['$finalizationRegistry'],
   $detachFinalizer: function(handle) {},
 
-  $attachFinalizer__deps: ['$finalizationGroup', '$detachFinalizer',
-                           '$releaseClassHandle'],
+  $attachFinalizer__deps: ['$finalizationRegistry', '$detachFinalizer',
+                           '$releaseClassHandle', '$RegisteredPointer_fromWireType'],
   $attachFinalizer: function(handle) {
-    if ('undefined' === typeof FinalizationGroup) {
-        attachFinalizer = function (handle) { return handle; };
+    if ('undefined' === typeof FinalizationRegistry) {
+        attachFinalizer = (handle) => handle;
         return handle;
     }
-    // If the running environment has a FinalizationGroup (see
+    // If the running environment has a FinalizationRegistry (see
     // https://github.com/tc39/proposal-weakrefs), then attach finalizers
-    // for class handles.  We check for the presence of FinalizationGroup
+    // for class handles.  We check for the presence of FinalizationRegistry
     // at run-time, not build-time.
-    finalizationGroup = new FinalizationGroup(function (iter) {
-        for (var result = iter.next(); !result.done; result = iter.next()) {
-            var $$ = result.value;
-            if (!$$.ptr) {
-                console.warn('object already deleted: ' + $$.ptr);
-            } else {
-                releaseClassHandle($$);
-            }
-        }
+    finalizationRegistry = new FinalizationRegistry((info) => {
+#if ASSERTIONS
+        console.warn(info.leakWarning.stack.replace(/^Error: /, ''));
+#endif
+        releaseClassHandle(info.$$);
     });
-    attachFinalizer = function(handle) {
-        finalizationGroup.register(handle, handle.$$, handle.$$);
+    attachFinalizer = (handle) => {
+        var $$ = handle.$$;
+        var hasSmartPtr = !!$$.smartPtr;
+        if (hasSmartPtr) {
+            // We should not call the destructor on raw pointers in case other code expects the pointee to live
+            var info = { $$: $$ };
+#if ASSERTIONS
+            // Create a warning as an Error instance in advance so that we can store
+            // the current stacktrace and point to it when / if a leak is detected.
+            // This is more useful than the empty stacktrace of `FinalizationRegistry`
+            // callback.
+            var cls = $$.ptrType.registeredClass;
+            info.leakWarning = new Error("Embind found a leaked C++ instance " + cls.name + " <0x" + $$.ptr.toString(16) + ">.\n" +
+            "We'll free it automatically in this case, but this functionality is not reliable across various environments.\n" +
+            "Make sure to invoke .delete() manually once you're done with the instance instead.\n" +
+            "Originally allocated"); // `.stack` will add "at ..." after this sentence
+            if ('captureStackTrace' in Error) {
+                Error.captureStackTrace(info.leakWarning, RegisteredPointer_fromWireType);
+            }
+ #endif
+            finalizationRegistry.register(handle, info, handle);
+        }
         return handle;
     };
-    detachFinalizer = function(handle) {
-        finalizationGroup.unregister(handle.$$);
-    };
+    detachFinalizer = (handle) => finalizationRegistry.unregister(handle);
     return attachFinalizer(handle);
   },
 
@@ -1981,7 +2089,7 @@ var LibraryEmbind = {
 
   _embind_register_class_constructor__deps: [
     '$heap32VectorToArray', '$embind__requireFunction', '$runDestructors',
-    '$throwBindingError', '$whenDependentTypesAreResolved'],
+    '$throwBindingError', '$whenDependentTypesAreResolved', '$registeredTypes'],
   _embind_register_class_constructor: function(
     rawClassType,
     argCount,
@@ -2006,26 +2114,14 @@ var LibraryEmbind = {
         if (undefined !== classType.registeredClass.constructor_body[argCount - 1]) {
             throw new BindingError("Cannot register multiple constructors with identical number of parameters (" + (argCount-1) + ") for class '" + classType.name + "'! Overload resolution is currently only performed using the parameter count, not actual type info!");
         }
-        classType.registeredClass.constructor_body[argCount - 1] = function unboundTypeHandler() {
+        classType.registeredClass.constructor_body[argCount - 1] = () => {
             throwUnboundTypeError('Cannot construct ' + classType.name + ' due to unbound types', rawArgTypes);
         };
 
         whenDependentTypesAreResolved([], rawArgTypes, function(argTypes) {
-            classType.registeredClass.constructor_body[argCount - 1] = function constructor_body() {
-                if (arguments.length !== argCount - 1) {
-                    throwBindingError(humanName + ' called with ' + arguments.length + ' arguments, expected ' + (argCount-1));
-                }
-                destructors.length = 0;
-                args.length = argCount;
-                for (var i = 1; i < argCount; ++i) {
-                    args[i] = argTypes[i]['toWireType'](destructors, arguments[i - 1]);
-                }
-
-                var ptr = invoker.apply(null, args);
-                runDestructors(destructors);
-
-                return argTypes[0]['fromWireType'](ptr);
-            };
+            // Insert empty slot for context type (argTypes[1]).
+            argTypes.splice(1, 0, null);
+            classType.registeredClass.constructor_body[argCount - 1] = craftInvokerFunction(humanName, argTypes, null, invoker, rawConstructor);
             return [];
         });
         return [];
@@ -2100,6 +2196,10 @@ var LibraryEmbind = {
         classType = classType[0];
         var humanName = classType.name + '.' + methodName;
 
+        if (methodName.startsWith("@@")) {
+            methodName = Symbol[methodName.substring(2)];
+        }
+
         if (isPureVirtual) {
             classType.registeredClass.pureVirtualFunctions.push(methodName);
         }
@@ -2171,11 +2271,11 @@ var LibraryEmbind = {
             configurable: true
         };
         if (setter) {
-            desc.set = function() {
+            desc.set = () => {
                 throwUnboundTypeError('Cannot access ' + humanName + ' due to unbound types', [getterReturnType, setterArgumentType]);
             };
         } else {
-            desc.set = function(v) {
+            desc.set = (v) => {
                 throwBindingError(humanName + ' is a read-only property');
             };
         }
@@ -2238,6 +2338,10 @@ var LibraryEmbind = {
             throwUnboundTypeError('Cannot call ' + humanName + ' due to unbound types', rawArgTypes);
         }
 
+        if (methodName.startsWith("@@")) {
+            methodName = Symbol[methodName.substring(2)];
+        }
+
         var proto = classType.registeredClass.constructor;
         if (undefined === proto[methodName]) {
             // This is the first function to be registered with this name.
@@ -2294,11 +2398,11 @@ var LibraryEmbind = {
             configurable: true
         };
         if (setter) {
-            desc.set = function() {
+            desc.set = () => {
                 throwUnboundTypeError('Cannot access ' + humanName + ' due to unbound types', [rawFieldType]);
             };
         } else {
-            desc.set = function(v) {
+            desc.set = (v) => {
                 throwBindingError(humanName + ' is a read-only property');
             };
         }
@@ -2316,7 +2420,7 @@ var LibraryEmbind = {
 
             if (setter) {
                 setter = embind__requireFunction(setterSignature, setter);
-                desc.set = function(v) {
+                desc.set = (v) => {
                     var destructors = [];
                     setter(rawFieldPtr, fieldType['toWireType'](destructors, v));
                     runDestructors(destructors);
@@ -2332,15 +2436,15 @@ var LibraryEmbind = {
   },
 
   _embind_create_inheriting_constructor__deps: [
-    '$createNamedFunction', '_emval_register',
+    '$createNamedFunction', '$Emval',
     '$PureVirtualError', '$readLatin1String',
-    '$registerInheritedInstance', '$requireHandle',
+    '$registerInheritedInstance',
     '$requireRegisteredType', '$throwBindingError',
     '$unregisterInheritedInstance', '$detachFinalizer', '$attachFinalizer'],
   _embind_create_inheriting_constructor: function(constructorName, wrapperType, properties) {
     constructorName = readLatin1String(constructorName);
     wrapperType = requireRegisteredType(wrapperType, 'wrapper');
-    properties = requireHandle(properties);
+    properties = Emval.toValue(properties);
 
     var arraySlice = [].slice;
 
@@ -2396,7 +2500,7 @@ var LibraryEmbind = {
     for (var p in properties) {
         ctor.prototype[p] = properties[p];
     }
-    return __emval_register(ctor);
+    return Emval.toHandle(ctor);
   },
 
   $char_0: '0'.charCodeAt(0),
