@@ -9,20 +9,6 @@ var LibraryExceptions = {
   $exceptionLast: '0',
   $exceptionCaught: ' []',
 
-  // Static fields for ExceptionInfo class.
-  $ExceptionInfoAttrs: {
-    // ExceptionInfo native structure layout.
-    DESTRUCTOR_OFFSET: 0,
-    REFCOUNT_OFFSET: Runtime.POINTER_SIZE,
-    TYPE_OFFSET: Runtime.POINTER_SIZE + 4,
-    CAUGHT_OFFSET: Runtime.POINTER_SIZE + 8,
-    RETHROWN_OFFSET: Runtime.POINTER_SIZE + 9,
-
-    // Total structure size with padding, should be multiple of allocation alignment.
-    SIZE: alignMemory(Runtime.POINTER_SIZE + 10)
-  },
-
-  $ExceptionInfo__deps: ['$ExceptionInfoAttrs'],
   // This class is the exception metadata which is prepended to each thrown object (in WASM memory).
   // It is allocated in one block among with a thrown object in __cxa_allocate_exception and freed
   // in ___cxa_free_exception. It roughly corresponds to __cxa_exception structure in libcxxabi. The
@@ -35,46 +21,47 @@ var LibraryExceptions = {
   // purpose, it references the primary exception.
   //
   // excPtr - Thrown object pointer to wrap. Metadata pointer is calculated from it.
+  $ExceptionInfo__docs: '/** @constructor */',
   $ExceptionInfo: function(excPtr) {
     this.excPtr = excPtr;
-    this.ptr = excPtr - ExceptionInfoAttrs.SIZE;
+    this.ptr = excPtr - {{{ C_STRUCTS.__cxa_exception.__size__ }}};
 
     this.set_type = function(type) {
-      {{{ makeSetValue('this.ptr', 'ExceptionInfoAttrs.TYPE_OFFSET', 'type', '*') }}};
+      {{{ makeSetValue('this.ptr', C_STRUCTS.__cxa_exception.exceptionType, 'type', '*') }}};
     };
 
     this.get_type = function() {
-      return {{{ makeGetValue('this.ptr', 'ExceptionInfoAttrs.TYPE_OFFSET', '*') }}};
+      return {{{ makeGetValue('this.ptr', C_STRUCTS.__cxa_exception.exceptionType, '*') }}};
     };
 
     this.set_destructor = function(destructor) {
-      {{{ makeSetValue('this.ptr', 'ExceptionInfoAttrs.DESTRUCTOR_OFFSET', 'destructor', '*') }}};
+      {{{ makeSetValue('this.ptr', C_STRUCTS.__cxa_exception.exceptionDestructor, 'destructor', '*') }}};
     };
 
     this.get_destructor = function() {
-      return {{{ makeGetValue('this.ptr', 'ExceptionInfoAttrs.DESTRUCTOR_OFFSET', '*') }}};
+      return {{{ makeGetValue('this.ptr', C_STRUCTS.__cxa_exception.exceptionDestructor, '*') }}};
     };
 
     this.set_refcount = function(refcount) {
-      {{{ makeSetValue('this.ptr', 'ExceptionInfoAttrs.REFCOUNT_OFFSET', 'refcount', 'i32') }}};
+      {{{ makeSetValue('this.ptr', C_STRUCTS.__cxa_exception.referenceCount, 'refcount', 'i32') }}};
     };
 
     this.set_caught = function (caught) {
       caught = caught ? 1 : 0;
-      {{{ makeSetValue('this.ptr', 'ExceptionInfoAttrs.CAUGHT_OFFSET', 'caught', 'i8') }}};
+      {{{ makeSetValue('this.ptr', C_STRUCTS.__cxa_exception.caught, 'caught', 'i8') }}};
     };
 
     this.get_caught = function () {
-      return {{{ makeGetValue('this.ptr', 'ExceptionInfoAttrs.CAUGHT_OFFSET', 'i8') }}} != 0;
+      return {{{ makeGetValue('this.ptr', C_STRUCTS.__cxa_exception.caught, 'i8') }}} != 0;
     };
 
     this.set_rethrown = function (rethrown) {
       rethrown = rethrown ? 1 : 0;
-      {{{ makeSetValue('this.ptr', 'ExceptionInfoAttrs.RETHROWN_OFFSET', 'rethrown', 'i8') }}};
+      {{{ makeSetValue('this.ptr', C_STRUCTS.__cxa_exception.rethrown, 'rethrown', 'i8') }}};
     };
 
     this.get_rethrown = function () {
-      return {{{ makeGetValue('this.ptr', 'ExceptionInfoAttrs.RETHROWN_OFFSET', 'i8') }}} != 0;
+      return {{{ makeGetValue('this.ptr', C_STRUCTS.__cxa_exception.rethrown, 'i8') }}} != 0;
     };
 
     // Initialize native structure fields. Should be called once after allocated.
@@ -88,20 +75,20 @@ var LibraryExceptions = {
 
     this.add_ref = function() {
 #if USE_PTHREADS
-      Atomics.add(HEAP32, (this.ptr + ExceptionInfoAttrs.REFCOUNT_OFFSET) >> 2, 1);
+      Atomics.add(HEAP32, (this.ptr + {{{ C_STRUCTS.__cxa_exception.referenceCount }}}) >> 2, 1);
 #else
-      var value = {{{ makeGetValue('this.ptr', 'ExceptionInfoAttrs.REFCOUNT_OFFSET', 'i32') }}};
-      {{{ makeSetValue('this.ptr', 'ExceptionInfoAttrs.REFCOUNT_OFFSET', 'value + 1', 'i32') }}};
+      var value = {{{ makeGetValue('this.ptr', C_STRUCTS.__cxa_exception.referenceCount, 'i32') }}};
+      {{{ makeSetValue('this.ptr', C_STRUCTS.__cxa_exception.referenceCount, 'value + 1', 'i32') }}};
 #endif
     };
 
     // Returns true if last reference released.
     this.release_ref = function() {
 #if USE_PTHREADS
-      var prev = Atomics.sub(HEAP32, (this.ptr + ExceptionInfoAttrs.REFCOUNT_OFFSET) >> 2, 1);
+      var prev = Atomics.sub(HEAP32, (this.ptr + {{{ C_STRUCTS.__cxa_exception.referenceCount }}}) >> 2, 1);
 #else
-      var prev = {{{ makeGetValue('this.ptr', 'ExceptionInfoAttrs.REFCOUNT_OFFSET', 'i32') }}};
-      {{{ makeSetValue('this.ptr', 'ExceptionInfoAttrs.REFCOUNT_OFFSET', 'prev - 1', 'i32') }}};
+      var prev = {{{ makeGetValue('this.ptr', C_STRUCTS.__cxa_exception.referenceCount, 'i32') }}};
+      {{{ makeSetValue('this.ptr', C_STRUCTS.__cxa_exception.referenceCount, 'prev - 1', 'i32') }}};
 #endif
 #if ASSERTIONS
       assert(prev > 0);
@@ -110,7 +97,12 @@ var LibraryExceptions = {
     };
   },
 
-  $CatchInfo__deps: ['$ExceptionInfo'],
+  $CatchInfo__docs: `
+  /**
+   * @constructor
+   * @param {number=} ptr
+   */`,
+  $CatchInfo__deps: ['$ExceptionInfo', '__cxa_is_pointer_type'],
   // This native structure is returned from __cxa_find_matching_catch, and serves as catching
   // context, i.e. stores information required to proceed with a specific selected catch. It stores
   // base and adjusted pointers of a thrown object. It is allocated dynamically and should be freed
@@ -134,13 +126,15 @@ var LibraryExceptions = {
     };
 
     this.set_adjusted_ptr = function(adjustedPtr) {
-      var ptrSize = {{{ Runtime.POINTER_SIZE }}};
-      {{{ makeSetValue('this.ptr', 'ptrSize', 'adjustedPtr', '*') }}};
+      {{{ makeSetValue('this.ptr', Runtime.POINTER_SIZE, 'adjustedPtr', '*') }}};
     };
 
+    this.get_adjusted_ptr_addr = function() {
+      return this.ptr + {{{ Runtime.POINTER_SIZE }}};
+    }
+
     this.get_adjusted_ptr = function() {
-      var ptrSize = {{{ Runtime.POINTER_SIZE }}};
-      return {{{ makeGetValue('this.ptr', 'ptrSize', '*') }}};
+      return {{{ makeGetValue('this.ptr', Runtime.POINTER_SIZE, '*') }}};
     };
 
     // Get pointer which is expected to be received by catch clause in C++ code. It may be adjusted
@@ -205,13 +199,14 @@ var LibraryExceptions = {
   },
 
   // Exceptions
-  __cxa_allocate_exception__deps: ['$ExceptionInfoAttrs'],
+  __cxa_allocate_exception__sig: 'vi',
   __cxa_allocate_exception: function(size) {
     // Thrown object is prepended by exception metadata block
-    return _malloc(size + ExceptionInfoAttrs.SIZE) + ExceptionInfoAttrs.SIZE;
+    return _malloc(size + {{{ C_STRUCTS.__cxa_exception.__size__ }}}) + {{{ C_STRUCTS.__cxa_exception.__size__ }}};
   },
 
   __cxa_free_exception__deps: ['$ExceptionInfo'],
+  __cxa_free_exception__sig: 'vi',
   __cxa_free_exception: function(ptr) {
 #if ABORTING_MALLOC || ASSERTIONS
     try {
@@ -380,7 +375,7 @@ var LibraryExceptions = {
   // unwinding using 'if' blocks around each function, so the remaining
   // functionality boils down to picking a suitable 'catch' block.
   // We'll do that here, instead, to keep things simpler.
-  __cxa_find_matching_catch__deps: ['$exceptionLast', '$ExceptionInfo', '$CatchInfo', '__resumeException'],
+  __cxa_find_matching_catch__deps: ['$exceptionLast', '$ExceptionInfo', '$CatchInfo', '__resumeException', '__cxa_can_catch'],
   __cxa_find_matching_catch: function() {
     var thrown = exceptionLast;
     if (!thrown) {
@@ -391,6 +386,7 @@ var LibraryExceptions = {
     var thrownType = info.get_type();
     var catchInfo = new CatchInfo();
     catchInfo.set_base_ptr(thrown);
+    catchInfo.set_adjusted_ptr(thrown);
     if (!thrownType) {
       // just pass through the thrown ptr
       {{{ makeStructuralReturn(['catchInfo.ptr', 0]) }}};
@@ -401,9 +397,6 @@ var LibraryExceptions = {
 #if EXCEPTION_DEBUG
     out("can_catch on " + [thrown]);
 #endif
-    var stackTop = stackSave();
-    var exceptionThrowBuf = stackAlloc(4);
-    {{{ makeSetValue('exceptionThrowBuf', '0', 'thrown', '*') }}};
     // The different catch blocks are denoted by different types.
     // Due to inheritance, those types may not precisely match the
     // type of the thrown object. Find one which matches, and
@@ -414,18 +407,13 @@ var LibraryExceptions = {
         // Catch all clause matched or exactly the same type is caught
         break;
       }
-      if ({{{ exportedAsmFunc('___cxa_can_catch') }}}(caughtType, thrownType, exceptionThrowBuf)) {
-        var adjusted = {{{ makeGetValue('exceptionThrowBuf', '0', '*') }}};
-        if (thrown !== adjusted) {
-          catchInfo.set_adjusted_ptr(adjusted);
-        }
+      if ({{{ exportedAsmFunc('___cxa_can_catch') }}}(caughtType, thrownType, catchInfo.get_adjusted_ptr_addr())) {
 #if EXCEPTION_DEBUG
-        out("  can_catch found " + [adjusted, caughtType]);
+        out("  can_catch found " + [catchInfo.get_adjusted_ptr(), caughtType]);
 #endif
         {{{ makeStructuralReturn(['catchInfo.ptr', 'caughtType']) }}};
       }
     }
-    stackRestore(stackTop);
     {{{ makeStructuralReturn(['catchInfo.ptr', 'thrownType']) }}};
   },
 
