@@ -80,8 +80,9 @@ var LibraryGLFW = {
     },
 
   $GLFW__deps: ['emscripten_get_now', '$GL', '$Browser', '$GLFW_Window',
+    '$callUserCallback',
 #if FILESYSTEM
-    , '$FS'
+    '$FS',
 #endif
   ],
   $GLFW: {
@@ -191,7 +192,7 @@ var LibraryGLFW = {
         case 0xDB:return 91; // DOM_VK_OPEN_BRACKET -> GLFW_KEY_LEFT_BRACKET
         case 0xDC:return 92; // DOM_VK_BACKSLASH -> GLFW_KEY_BACKSLASH
         case 0xDD:return 93; // DOM_VK_CLOSE_BRACKET -> GLFW_KEY_RIGHT_BRACKET
-        case 0xC0:return 94; // DOM_VK_BACK_QUOTE -> GLFW_KEY_GRAVE_ACCENT
+        case 0xC0:return 96; // DOM_VK_BACK_QUOTE -> GLFW_KEY_GRAVE_ACCENT
 
 #if USE_GLFW == 2
         //#define GLFW_KEY_SPECIAL      256
@@ -591,13 +592,15 @@ var LibraryGLFW = {
 
       if (!GLFW.active.windowSizeFunc) return;
 
+      callUserCallback(function() {
 #if USE_GLFW == 2
-      {{{ makeDynCall('vii', 'GLFW.active.windowSizeFunc') }}}(GLFW.active.width, GLFW.active.height);
+        {{{ makeDynCall('vii', 'GLFW.active.windowSizeFunc') }}}(GLFW.active.width, GLFW.active.height);
 #endif
 
 #if USE_GLFW == 3
-      {{{ makeDynCall('viii', 'GLFW.active.windowSizeFunc') }}}(GLFW.active.id, GLFW.active.width, GLFW.active.height);
+        {{{ makeDynCall('viii', 'GLFW.active.windowSizeFunc') }}}(GLFW.active.id, GLFW.active.width, GLFW.active.height);
 #endif
+      });
     },
 
     onFramebufferSizeChanged: function() {
@@ -605,9 +608,11 @@ var LibraryGLFW = {
 
       if (!GLFW.active.framebufferSizeFunc) return;
 
+      callUserCallback(function() {
 #if USE_GLFW == 3
-      {{{ makeDynCall('viii', 'GLFW.active.framebufferSizeFunc') }}}(GLFW.active.id, GLFW.active.width, GLFW.active.height);
+        {{{ makeDynCall('viii', 'GLFW.active.framebufferSizeFunc') }}}(GLFW.active.id, GLFW.active.width, GLFW.active.height);
 #endif
+      });
     },
 
     getTime: function() {
@@ -632,13 +637,13 @@ var LibraryGLFW = {
     },
 
     joys: {}, // glfw joystick data
-    lastGamepadState: null,
+    lastGamepadState: [],
     lastGamepadStateFrame: null, // The integer value of Browser.mainLoop.currentFrameNumber of when the last gamepad state was produced.
 
     refreshJoysticks: function() {
       // Produce a new Gamepad API sample if we are ticking a new game frame, or if not using emscripten_set_main_loop() at all to drive animation.
       if (Browser.mainLoop.currentFrameNumber !== GLFW.lastGamepadStateFrame || !Browser.mainLoop.currentFrameNumber) {
-        GLFW.lastGamepadState = navigator.getGamepads ? navigator.getGamepads() : (navigator.webkitGetGamepads ? navigator.webkitGetGamepads : null);
+        GLFW.lastGamepadState = navigator.getGamepads ? navigator.getGamepads() : (navigator.webkitGetGamepads ? navigator.webkitGetGamepads : []);
         GLFW.lastGamepadStateFrame = Browser.mainLoop.currentFrameNumber;
 
         for (var joy = 0; joy < GLFW.lastGamepadState.length; ++joy) {
@@ -646,13 +651,13 @@ var LibraryGLFW = {
 
           if (gamepad) {
             if (!GLFW.joys[joy]) {
-              console.log('glfw joystick connected:',joy);
+              out('glfw joystick connected:',joy);
               GLFW.joys[joy] = {
-                id: allocate(intArrayFromString(gamepad.id), ALLOC_NORMAL),
+                id: allocateUTF8(gamepad.id),
                 buttonsCount: gamepad.buttons.length,
                 axesCount: gamepad.axes.length,
-                buttons: allocate(new Array(gamepad.buttons.length), ALLOC_NORMAL),
-                axes: allocate(new Array(gamepad.axes.length*4), ALLOC_NORMAL)
+                buttons: _malloc(gamepad.buttons.length),
+                axes: _malloc(gamepad.axes.length*4),
               };
 
               if (GLFW.joystickFunc) {
@@ -671,7 +676,7 @@ var LibraryGLFW = {
             }
           } else {
             if (GLFW.joys[joy]) {
-              console.log('glfw joystick disconnected',joy);
+              out('glfw joystick disconnected',joy);
 
               if (GLFW.joystickFunc) {
                 {{{ makeDynCall('vii', 'GLFW.joystickFunc') }}}(joy, 0x00040002); // GLFW_DISCONNECTED
@@ -743,7 +748,7 @@ var LibraryGLFW = {
       event.preventDefault();
 
 #if FILESYSTEM
-      var filenames = allocate(new Array(event.dataTransfer.files.length*4), ALLOC_NORMAL);
+      var filenames = _malloc(event.dataTransfer.files.length*4);
       var filenamesArray = [];
       var count = event.dataTransfer.files.length;
 
@@ -755,10 +760,10 @@ var LibraryGLFW = {
       function save(file) {
         var path = '/' + drop_dir + '/' + file.name.replace(/\//g, '_');
         var reader = new FileReader();
-        reader.onloadend = function(e) {
+        reader.onloadend = (e) => {
           if (reader.readyState != 2) { // not DONE
             ++written;
-            console.log('failed to read dropped file: '+file.name+': '+reader.error);
+            out('failed to read dropped file: '+file.name+': '+reader.error);
             return;
           }
 
@@ -775,7 +780,7 @@ var LibraryGLFW = {
         };
         reader.readAsArrayBuffer(file);
 
-        var filename = allocate(intArrayFromString(path), ALLOC_NORMAL);
+        var filename = allocateUTF8(path);
         filenamesArray.push(filename);
         setValue(filenames + i*4, filename, 'i8*');
       }
@@ -840,9 +845,9 @@ var LibraryGLFW = {
       var win = GLFW.WindowFromId(winid);
       if (!win) return;
 
-      switch(mode) {
+      switch (mode) {
         case 0x00033001: { // GLFW_CURSOR
-          switch(value) {
+          switch (value) {
             case 0x00034001: { // GLFW_CURSOR_NORMAL
               win.inputModes[mode] = value;
               Module['canvas'].removeEventListener('click', GLFW.onClickRequestPointerLock, true);
@@ -850,7 +855,7 @@ var LibraryGLFW = {
               break;
             }
             case 0x00034002: { // GLFW_CURSOR_HIDDEN
-              console.log("glfwSetInputMode called with GLFW_CURSOR_HIDDEN value not implemented.");
+              out("glfwSetInputMode called with GLFW_CURSOR_HIDDEN value not implemented.");
               break;
             }
             case 0x00034003: { // GLFW_CURSOR_DISABLED
@@ -860,22 +865,22 @@ var LibraryGLFW = {
               break;
             }
             default: {
-              console.log("glfwSetInputMode called with unknown value parameter value: " + value + ".");
+              out("glfwSetInputMode called with unknown value parameter value: " + value + ".");
               break;
             }
           }
           break;
         }
         case 0x00033002: { // GLFW_STICKY_KEYS
-          console.log("glfwSetInputMode called with GLFW_STICKY_KEYS mode not implemented.");
+          out("glfwSetInputMode called with GLFW_STICKY_KEYS mode not implemented.");
           break;
         }
         case 0x00033003: { // GLFW_STICKY_MOUSE_BUTTONS
-          console.log("glfwSetInputMode called with GLFW_STICKY_MOUSE_BUTTONS mode not implemented.");
+          out("glfwSetInputMode called with GLFW_STICKY_MOUSE_BUTTONS mode not implemented.");
           break;
         }
         default: {
-          console.log("glfwSetInputMode called with unknown mode parameter value: " + mode + ".");
+          out("glfwSetInputMode called with unknown mode parameter value: " + mode + ".");
           break;
         }
       }
@@ -1099,7 +1104,7 @@ var LibraryGLFW = {
 /*******************************************************************************
  * GLFW FUNCTIONS
  ******************************************************************************/
-
+  glfwInit__sig: 'i',
   glfwInit: function() {
     if (GLFW.windows) return 1; // GL_TRUE
 
@@ -1134,6 +1139,7 @@ var LibraryGLFW = {
     return 1; // GL_TRUE
   },
 
+  glfwTerminate__sig: 'v',
   glfwTerminate: function() {
     window.removeEventListener("gamepadconnected", GLFW.onGamepadConnected, true);
     window.removeEventListener("gamepaddisconnected", GLFW.onGamepadDisconnected, true);
@@ -1161,6 +1167,7 @@ var LibraryGLFW = {
     GLFW.active = null;
   },
 
+  glfwGetVersion__sig: 'viii',
   glfwGetVersion: function(major, minor, rev) {
 #if USE_GLFW == 2
     setValue(major, 2, 'i32');
@@ -1175,37 +1182,38 @@ var LibraryGLFW = {
 #endif
   },
 
+  glfwPollEvents__sig: 'v',
   glfwPollEvents: function() {},
 
+  glfwWaitEvents__sig: 'v',
   glfwWaitEvents: function() {},
 
+  glfwGetTime__sig: 'd',
   glfwGetTime: function() {
     return GLFW.getTime() - GLFW.initialTime;
   },
 
+  glfwSetTime__sig: 'vd',
   glfwSetTime: function(time) {
     GLFW.initialTime = GLFW.getTime() - time;
   },
 
   glfwExtensionSupported__deps: ['glGetString'],
+  glfwExtensionSupported__sig: 'ii',
   glfwExtensionSupported: function(extension) {
     if (!GLFW.extensions) {
       GLFW.extensions = UTF8ToString(_glGetString(0x1F03)).split(' ');
     }
 
-    if (GLFW.extensions.indexOf(extension) != -1) return 1;
+    if (GLFW.extensions.includes(extension)) return 1;
 
     // extensions from GLEmulations do not come unprefixed
     // so, try with prefix
-    return (GLFW.extensions.indexOf("GL_" + extension) != -1);
-  },
-
-  glfwGetProcAddress__deps: ['emscripten_GetProcAddress'],
-  glfwGetProcAddress: function(procname) {
-    return _emscripten_GetProcAddress(procname);
+    return (GLFW.extensions.includes("GL_" + extension));
   },
 
   glfwSwapInterval__deps: ['emscripten_set_main_loop_timing'],
+  glfwSwapInterval__sig: 'vi',
   glfwSwapInterval: function(interval) {
     interval = Math.abs(interval); // GLFW uses negative values to enable GLX_EXT_swap_control_tear, which we don't have, so just treat negative and positive the same.
     if (interval == 0) _emscripten_set_main_loop_timing(0/*EM_TIMING_SETTIMEOUT*/, 0);
@@ -1213,23 +1221,28 @@ var LibraryGLFW = {
   },
 
 #if USE_GLFW == 3
+  glfwGetVersionString__sig: 'i',
   glfwGetVersionString: function() {
     if (!GLFW.versionString) {
-      GLFW.versionString = allocate(intArrayFromString("3.2.1 JS WebGL Emscripten"), ALLOC_NORMAL);
+      GLFW.versionString = allocateUTF8("3.2.1 JS WebGL Emscripten");
     }
     return GLFW.versionString;
   },
 
+  glfwSetErrorCallback__sig: 'ii',
   glfwSetErrorCallback: function(cbfun) {
     var prevcbfun = GLFW.errorFunc;
     GLFW.errorFunc = cbfun;
     return prevcbfun;
   },
 
+  glfwWaitEventsTimeout__sig: 'vd',
   glfwWaitEventsTimeout: function(timeout) {},
 
+  glfwPostEmptyEvent__sig: 'v',
   glfwPostEmptyEvent: function() {},
 
+  glfwGetMonitors__sig: 'ii',
   glfwGetMonitors: function(count) {
     setValue(count, 1, 'i32');
     if (!GLFW.monitors) {
@@ -1239,15 +1252,18 @@ var LibraryGLFW = {
     return GLFW.monitors;
   },
 
+  glfwGetPrimaryMonitor__sig: 'i',
   glfwGetPrimaryMonitor: function() {
     return 1;
   },
 
+  glfwGetMonitorPos__sig: 'viii',
   glfwGetMonitorPos: function(monitor, x, y) {
     setValue(x, 0, 'i32');
     setValue(y, 0, 'i32');
   },
 
+  glfwGetMonitorPhysicalSize__sig: 'viii',
   glfwGetMonitorPhysicalSize: function(monitor, width, height) {
     // AFAIK there is no way to do this in javascript
     // Maybe with platform specific ccalls?
@@ -1257,13 +1273,15 @@ var LibraryGLFW = {
     setValue(height, 0, 'i32');
   },
 
+  glfwGetMonitorName__sig: 'ii',
   glfwGetMonitorName: function(mon) {
     if (!GLFW.monitorString) {
-      GLFW.monitorString = allocate(intArrayFromString("HTML5 WebGL Canvas"), ALLOC_NORMAL);
+      GLFW.monitorString = allocateUTF8("HTML5 WebGL Canvas");
     }
     return GLFW.monitorString;
   },
 
+  glfwSetMonitorCallback__sig: 'ii',
   glfwSetMonitorCallback: function(cbfun) {
     var prevcbfun = GLFW.monitorFunc;
     GLFW.monitorFunc = cbfun;
@@ -1271,73 +1289,90 @@ var LibraryGLFW = {
   },
 
   // TODO: implement
+  glfwGetVideoModes__sig: 'iii',
   glfwGetVideoModes: function(monitor, count) {
     setValue(count, 0, 'i32');
     return 0;
   },
 
   // TODO: implement
+  glfwGetVideoMode__sig: 'ii',
   glfwGetVideoMode: function(monitor) { return 0; },
 
   // TODO: implement
+  glfwSetGamma__sig: 'vif',
   glfwSetGamma: function(monitor, gamma) { },
 
+  glfwGetGammaRamp__sig: 'ii',
   glfwGetGammaRamp: function(monitor) {
     throw "glfwGetGammaRamp not implemented.";
   },
 
+  glfwSetGammaRamp__sig: 'vii',
   glfwSetGammaRamp: function(monitor, ramp) {
     throw "glfwSetGammaRamp not implemented.";
   },
 
+  glfwDefaultWindowHints__sig: 'v',
   glfwDefaultWindowHints: function() {
     GLFW.hints = GLFW.defaultHints;
   },
 
+  glfwWindowHint__sig: 'vii',
   glfwWindowHint: function(target, hint) {
     GLFW.hints[target] = hint;
   },
 
+  glfwCreateWindow__sig: 'iiiiii',
   glfwCreateWindow: function(width, height, title, monitor, share) {
     return GLFW.createWindow(width, height, title, monitor, share);
   },
 
+  glfwDestroyWindow__sig: 'vi',
   glfwDestroyWindow: function(winid) {
     return GLFW.destroyWindow(winid);
   },
 
+  glfwWindowShouldClose__sig: 'ii',
   glfwWindowShouldClose: function(winid) {
     var win = GLFW.WindowFromId(winid);
     if (!win) return 0;
     return win.shouldClose;
   },
 
+  glfwSetWindowShouldClose__sig: 'vii',
   glfwSetWindowShouldClose: function(winid, value) {
     var win = GLFW.WindowFromId(winid);
     if (!win) return;
     win.shouldClose = value;
   },
 
+  glfwSetWindowTitle__sig: 'vii',
   glfwSetWindowTitle: function(winid, title) {
     GLFW.setWindowTitle(winid, title);
   },
 
+  glfwGetWindowPos__sig: 'viii',
   glfwGetWindowPos: function(winid, x, y) {
     GLFW.getWindowPos(winid, x, y);
   },
 
+  glfwSetWindowPos__sig: 'viii',
   glfwSetWindowPos: function(winid, x, y) {
     GLFW.setWindowPos(winid, x, y);
   },
 
+  glfwGetWindowSize__sig: 'viii',
   glfwGetWindowSize: function(winid, width, height) {
     GLFW.getWindowSize(winid, width, height);
   },
 
+  glfwSetWindowSize__sig: 'viii',
   glfwSetWindowSize: function(winid, width, height) {
     GLFW.setWindowSize(winid, width, height);
   },
 
+  glfwGetFramebufferSize__sig: 'viii',
   glfwGetFramebufferSize: function(winid, width, height) {
     var ww = 0;
     var wh = 0;
@@ -1357,46 +1392,55 @@ var LibraryGLFW = {
     }
   },
 
+  glfwIconifyWindow__sig: 'vi',
   glfwIconifyWindow: function(winid) {
 #if ASSERTIONS
     warnOnce('glfwIconifyWindow is not implemented');
 #endif
   },
 
+  glfwRestoreWindow__sig: 'vi',
   glfwRestoreWindow: function(winid) {
 #if ASSERTIONS
     warnOnce('glfwRestoreWindow is not implemented');
 #endif
   },
 
+  glfwShowWindow__sig: 'vi',
   glfwShowWindow: function(winid) {},
 
+  glfwHideWindow__sig: 'vi',
   glfwHideWindow: function(winid) {},
 
+  glfwGetWindowMonitor__sig: 'ii',
   glfwGetWindowMonitor: function(winid) {
     var win = GLFW.WindowFromId(winid);
     if (!win) return 0;
     return win.monitor;
   },
 
+  glfwGetWindowAttrib__sig: 'iii',
   glfwGetWindowAttrib: function(winid, attrib) {
     var win = GLFW.WindowFromId(winid);
     if (!win) return 0;
     return win.attributes[attrib];
   },
 
+  glfwSetWindowUserPointer__sig: 'vii',
   glfwSetWindowUserPointer: function(winid, ptr) {
     var win = GLFW.WindowFromId(winid);
     if (!win) return;
     win.userptr = ptr;
   },
 
+  glfwGetWindowUserPointer__sig: 'ii',
   glfwGetWindowUserPointer: function(winid) {
     var win = GLFW.WindowFromId(winid);
     if (!win) return 0;
     return win.userptr;
   },
 
+  glfwSetWindowPosCallback__sig: 'iii',
   glfwSetWindowPosCallback: function(winid, cbfun) {
     var win = GLFW.WindowFromId(winid);
     if (!win) return null;
@@ -1405,18 +1449,22 @@ var LibraryGLFW = {
     return prevcbfun;
   },
 
+  glfwSetWindowSizeCallback__sig: 'iii',
   glfwSetWindowSizeCallback: function(winid, cbfun) {
     return GLFW.setWindowSizeCallback(winid, cbfun);
   },
 
+  glfwSetWindowCloseCallback__sig: 'iii',
   glfwSetWindowCloseCallback: function(winid, cbfun) {
     return GLFW.setWindowCloseCallback(winid, cbfun);
   },
 
+  glfwSetWindowRefreshCallback__sig: 'iii',
   glfwSetWindowRefreshCallback: function(winid, cbfun) {
     return GLFW.setWindowRefreshCallback(winid, cbfun);
   },
 
+  glfwSetWindowFocusCallback__sig: 'iii',
   glfwSetWindowFocusCallback: function(winid, cbfun) {
     var win = GLFW.WindowFromId(winid);
     if (!win) return null;
@@ -1425,6 +1473,7 @@ var LibraryGLFW = {
     return prevcbfun;
   },
 
+  glfwSetWindowIconifyCallback__sig: 'iii',
   glfwSetWindowIconifyCallback: function(winid, cbfun) {
     var win = GLFW.WindowFromId(winid);
     if (!win) return null;
@@ -1433,28 +1482,40 @@ var LibraryGLFW = {
     return prevcbfun;
   },
 
+  glfwSetWindowIcon__sig: 'viii',
   glfwSetWindowIcon: function(winid, count, images) {},
 
+  glfwSetWindowSizeLimits__sig: 'viiiii',
   glfwSetWindowSizeLimits: function(winid, minwidth, minheight, maxwidth, maxheight) {},
 
+  glfwSetWindowAspectRatio__sig: 'viii',
   glfwSetWindowAspectRatio: function(winid, numer, denom) {},
 
+  glfwGetWindowFrameSize__sig: 'viiiii',
   glfwGetWindowFrameSize: function(winid, left, top, right, bottom) { throw "glfwGetWindowFrameSize not implemented."; },
 
+  glfwMaximizeWindow__sig: 'vi',
   glfwMaximizeWindow: function(winid) {},
 
+  glfwFocusWindow__sig: 'vi',
   glfwFocusWindow: function(winid) {},
 
+  glfwSetWindowMonitor__sig: 'viiiiiii',
   glfwSetWindowMonitor: function(winid, monitor, xpos, ypos, width, height, refreshRate) { throw "glfwSetWindowMonitor not implemented."; },
 
+  glfwCreateCursor__sig: 'iiii',
   glfwCreateCursor: function(image, xhot, yhot) {},
 
+  glfwCreateStandardCursor__sig: 'ii',
   glfwCreateStandardCursor: function(shape) {},
 
+  glfwDestroyCursor__sig: 'vi',
   glfwDestroyCursor: function(cursor) {},
 
+  glfwSetCursor__sig: 'vii',
   glfwSetCursor: function(winid, cursor) {},
 
+  glfwSetFramebufferSizeCallback__sig: 'iii',
   glfwSetFramebufferSizeCallback: function(winid, cbfun) {
     var win = GLFW.WindowFromId(winid);
     if (!win) return null;
@@ -1463,6 +1524,7 @@ var LibraryGLFW = {
     return prevcbfun;
   },
 
+  glfwGetInputMode__sig: 'iii',
   glfwGetInputMode: function(winid, mode) {
     var win = GLFW.WindowFromId(winid);
     if (!win) return;
@@ -1480,47 +1542,59 @@ var LibraryGLFW = {
     return win.inputModes[mode];
   },
 
+  glfwSetInputMode__sig: 'viii',
   glfwSetInputMode: function(winid, mode, value) {
     GLFW.setInputMode(winid, mode, value);
   },
 
+  glfwGetKey__sig: 'iii',
   glfwGetKey: function(winid, key) {
     return GLFW.getKey(winid, key);
   },
 
+  glfwGetKeyName__sig: 'iii',
   glfwGetKeyName: function(key, scancode) { throw "glfwGetKeyName not implemented."; },
 
+  glfwGetMouseButton__sig: 'iii',
   glfwGetMouseButton: function(winid, button) {
     return GLFW.getMouseButton(winid, button);
   },
 
+  glfwGetCursorPos__sig: 'viii',
   glfwGetCursorPos: function(winid, x, y) {
     GLFW.getCursorPos(winid, x, y);
   },
 
   // I believe it is not possible to move the mouse with javascript
+  glfwSetCursorPos__sig: 'vidd',
   glfwSetCursorPos: function(winid, x, y) {
     GLFW.setCursorPos(winid, x, y);
   },
 
+  glfwSetKeyCallback__sig: 'iii',
   glfwSetKeyCallback: function(winid, cbfun) {
     return GLFW.setKeyCallback(winid, cbfun);
   },
 
+  glfwSetCharCallback__sig: 'iii',
   glfwSetCharCallback: function(winid, cbfun) {
     return GLFW.setCharCallback(winid, cbfun);
   },
 
+  glfwSetCharModsCallback__sig: 'iii',
   glfwSetCharModsCallback: function(winid, cbfun) { throw "glfwSetCharModsCallback not implemented."; },
 
+  glfwSetMouseButtonCallback__sig: 'iii',
   glfwSetMouseButtonCallback: function(winid, cbfun) {
     return GLFW.setMouseButtonCallback(winid, cbfun);
   },
 
+  glfwSetCursorPosCallback__sig: 'iii',
   glfwSetCursorPosCallback: function(winid, cbfun) {
     return GLFW.setCursorPosCallback(winid, cbfun);
   },
 
+  glfwSetCursorEnterCallback__sig: 'iii',
   glfwSetCursorEnterCallback: function(winid, cbfun) {
     var win = GLFW.WindowFromId(winid);
     if (!win) return null;
@@ -1529,36 +1603,47 @@ var LibraryGLFW = {
     return prevcbfun;
   },
 
+  glfwSetScrollCallback__sig: 'iii',
   glfwSetScrollCallback: function(winid, cbfun) {
     return GLFW.setScrollCallback(winid, cbfun);
   },
 
+  glfwVulkanSupported__sig: 'i',
   glfwVulkanSupported: function() {
     return 0;
   },
 
+  glfwSetDropCallback__sig: 'iii',
   glfwSetDropCallback: function(winid, cbfun) {
     return GLFW.setDropCallback(winid, cbfun);
   },
 
+  glfwGetTimerValue__sig: 'j',
   glfwGetTimerValue: function() { throw "glfwGetTimerValue is not implemented."; },
 
+  glfwGetTimerFrequency__sig: 'j',
   glfwGetTimerFrequency: function() { throw "glfwGetTimerFrequency is not implemented."; },
 
+  glfwGetRequiredInstanceExtensions__sig: 'ii',
   glfwGetRequiredInstanceExtensions: function(count) { throw "glfwGetRequiredInstanceExtensions is not implemented."; },
 
+  glfwGetInstanceProcAddress__sig: 'iii',
   glfwGetInstanceProcAddress: function(instance, procname) { throw "glfwGetInstanceProcAddress is not implemented."; },
 
+  glfwGetPhysicalDevicePresentationSupport__sig: 'iiii',
   glfwGetPhysicalDevicePresentationSupport: function(instance, device, queuefamily) { throw "glfwGetPhysicalDevicePresentationSupport is not implemented"; },
 
+  glfwCreateWindowSurface__sig: 'iiiii',
   glfwCreateWindowSurface: function(instance, winid, allocator, surface) { throw "glfwCreateWindowSurface is not implemented."; },
 
+  glfwJoystickPresent__sig: 'ii',
   glfwJoystickPresent: function(joy) {
     GLFW.refreshJoysticks();
 
     return GLFW.joys[joy] !== undefined;
   },
 
+  glfwGetJoystickAxes__sig: 'iii',
   glfwGetJoystickAxes: function(joy, count) {
     GLFW.refreshJoysticks();
 
@@ -1572,6 +1657,7 @@ var LibraryGLFW = {
     return state.axes;
   },
 
+  glfwGetJoystickButtons__sig: 'iii',
   glfwGetJoystickButtons: function(joy, count) {
     GLFW.refreshJoysticks();
 
@@ -1585,6 +1671,7 @@ var LibraryGLFW = {
     return state.buttons;
   },
 
+  glfwGetJoystickName__sig: 'ii',
   glfwGetJoystickName: function(joy) {
     if (GLFW.joys[joy]) {
       return GLFW.joys[joy].id;
@@ -1593,20 +1680,26 @@ var LibraryGLFW = {
     }
   },
 
+  glfwSetJoystickCallback__sig: 'ii',
   glfwSetJoystickCallback: function(cbfun) {
     GLFW.setJoystickCallback(cbfun);
   },
 
+  glfwSetClipboardString__sig: 'vii',
   glfwSetClipboardString: function(win, string) {},
 
+  glfwGetClipboardString__sig: 'ii',
   glfwGetClipboardString: function(win) {},
 
+  glfwMakeContextCurrent__sig: 'vi',
   glfwMakeContextCurrent: function(winid) {},
 
+  glfwGetCurrentContext__sig: 'i',
   glfwGetCurrentContext: function() {
     return GLFW.active ? GLFW.active.id : 0;
   },
 
+  glfwSwapBuffers__sig: 'vi',
   glfwSwapBuffers: function(winid) {
     GLFW.swapBuffers(winid);
   },
@@ -1756,7 +1849,7 @@ var LibraryGLFW = {
   },
 
   glfwCreateThread: function(fun, arg) {
-    {{{ makeDynCall('vi', 'str') }}}(fun, arg);
+    {{{ makeDynCall('vi', 'fun') }}}(arg);
     // One single thread
     return 0;
   },
