@@ -7,7 +7,11 @@
 
 #pragma once
 
+#include <stddef.h>
+#include <stdint.h>
 #include <emscripten/emscripten.h>
+#include <stdint.h>
+#include <stddef.h>
 
 #define WASM_PAGE_SIZE 65536
 #define EMSCRIPTEN_PAGE_SIZE WASM_PAGE_SIZE
@@ -18,7 +22,7 @@ extern "C" {
 
 // Returns a pointer to a memory location that contains the heap DYNAMICTOP
 // variable (the end of the dynamic memory region)
-intptr_t *emscripten_get_sbrk_ptr(void);
+uintptr_t *emscripten_get_sbrk_ptr(void);
 
 // Attempts to geometrically or linearly increase the heap so that it
 // grows by at least requested_growth_bytes new bytes. The heap size may
@@ -30,6 +34,9 @@ int emscripten_resize_heap(size_t requested_size) EM_IMPORT(emscripten_resize_he
 // Returns the current size of the WebAssembly heap.
 size_t emscripten_get_heap_size(void);
 
+// Returns the max size of the WebAssembly heap.
+size_t emscripten_get_heap_max(void);
+
 #ifdef __cplusplus
 }
 #endif
