@@ -1,9 +1,4 @@
-#include "libc.h"
-
-#ifdef __EMSCRIPTEN__
-#include <stdlib.h>
-#include <wasi/api.h>
-#endif
+#include <unistd.h>
 
 char **__environ = 0;
 weak_alias(__environ, ___environ);
@@ -11,6 +6,14 @@ weak_alias(__environ, _environ);
 weak_alias(__environ, environ);
 
 #ifdef __EMSCRIPTEN__
+#include <stdlib.h>
+#include <wasi/api.h>
+// Included for emscripten_builtin_free / emscripten_builtin_malloc
+// TODO(sbc): Should these be in their own header to avoid emmalloc here?
+#include <emscripten/emmalloc.h>
+
+// We use emscripten_builtin_malloc here because this memory is never freed and
+// and we don't want LSan to consider this a leak.
 __attribute__((constructor(100))) // construct this before user code
 void __emscripten_environ_constructor(void) {
     size_t environ_count;
@@ -21,11 +24,11 @@ void __emscripten_environ_constructor(void) {
         return;
     }
 
-    __environ = malloc(sizeof(char *) * (environ_count + 1));
+    __environ = emscripten_builtin_malloc(sizeof(char *) * (environ_count + 1));
     if (__environ == 0) {
         return;
     }
-    char *environ_buf = malloc(sizeof(char) * environ_buf_size);
+    char *environ_buf = emscripten_builtin_malloc(sizeof(char) * environ_buf_size);
     if (environ_buf == 0) {
         __environ = 0;
         return;
