@@ -1,9 +1,8 @@
 #include <sys/statvfs.h>
 #include <sys/statfs.h>
 #include "syscall.h"
-#include "libc.h"
 
-int __statfs(const char *path, struct statfs *buf)
+static int __statfs(const char *path, struct statfs *buf)
 {
 	*buf = (struct statfs){0};
 #ifdef SYS_statfs64
@@ -13,7 +12,7 @@ int __statfs(const char *path, struct statfs *buf)
 #endif
 }
 
-int __fstatfs(int fd, struct statfs *buf)
+static int __fstatfs(int fd, struct statfs *buf)
 {
 	*buf = (struct statfs){0};
 #ifdef SYS_fstatfs64
@@ -58,7 +57,7 @@ int fstatvfs(int fd, struct statvfs *buf)
 	return 0;
 }
 
-LFS64(statvfs);
-LFS64(statfs);
-LFS64(fstatvfs);
-LFS64(fstatfs);
+weak_alias(statvfs, statvfs64);
+weak_alias(statfs, statfs64);
+weak_alias(fstatvfs, fstatvfs64);
+weak_alias(fstatfs, fstatfs64);
