@@ -1,4 +1,5 @@
 #include "pthread_impl.h"
+#include "lock.h"
 
 int pthread_getschedparam(pthread_t t, int *restrict policy, struct sched_param *restrict param)
 {
@@ -7,8 +8,10 @@ int pthread_getschedparam(pthread_t t, int *restrict policy, struct sched_param
 	return 0;
 #else
 	int r;
-	__lock(t->killlock);
-	if (t->dead) {
+	sigset_t set;
+	__block_app_sigs(&set);
+	LOCK(t->killlock);
+	if (!t->tid) {
 		r = ESRCH;
 	} else {
 		r = -__syscall(SYS_sched_getparam, t->tid, param);
@@ -16,7 +19,8 @@ int pthread_getschedparam(pthread_t t, int *restrict policy, struct sched_param
 			*policy = __syscall(SYS_sched_getscheduler, t->tid);
 		}
 	}
-	__unlock(t->killlock);
+	UNLOCK(t->killlock);
+	__restore_sigs(&set);
 	return r;
 #endif
 }
