@@ -6,8 +6,6 @@
 #include <fcntl.h>
 #include "syscall.h"
 
-void __procfdname(char *, unsigned);
-
 int fchown(int fd, uid_t uid, gid_t gid)
 {
 	int ret = __syscall(SYS_fchown, fd, uid, gid);
@@ -24,7 +22,7 @@ int fchown(int fd, uid_t uid, gid_t gid)
 #ifdef SYS_chown
 	return syscall(SYS_chown, buf, uid, gid);
 #else
-	return syscall(SYS_fchownat, AT_FDCWD, buf, uid, gid);
+	return syscall(SYS_fchownat, AT_FDCWD, buf, uid, gid, 0);
 #endif
 
 }
