@@ -1,6 +1,5 @@
 #include <unistd.h>
 #include "syscall.h"
-#include "libc.h"
 
 ssize_t pread(int fd, void *buf, size_t size, off_t ofs)
 {
@@ -15,8 +14,8 @@ ssize_t pread(int fd, void *buf, size_t size, off_t ofs)
 	}
 	return num;
 #else
-	return syscall_cp(SYS_pread, fd, buf, size, __SYSCALL_LL_O(ofs));
+	return syscall_cp(SYS_pread, fd, buf, size, __SYSCALL_LL_PRW(ofs));
 #endif
 }
 
-LFS64(pread);
+weak_alias(pread, pread64);
