@@ -2,7 +2,6 @@
 #include <sys/uio.h>
 #include <unistd.h>
 #include "syscall.h"
-#include "libc.h"
 
 ssize_t preadv(int fd, const struct iovec *iov, int count, off_t ofs)
 {
@@ -18,4 +17,4 @@ ssize_t preadv(int fd, const struct iovec *iov, int count, off_t ofs)
 #endif
 }
 
-LFS64(preadv);
+weak_alias(preadv, preadv64);
