@@ -35,7 +35,5 @@ int main()
 {
   read_file();
   read_file();
-#ifdef REPORT_RESULT
-  REPORT_RESULT(0);
-#endif
+  return 0;
 }
