@@ -9,13 +9,14 @@
 #include <stdint.h>
 #include <stdio.h>
 
-void waka(int w, long long xy, int z) {
+int waka(int w, long long xy, int z) {
   // xy should be 0xffff_ffff_0000_0004
   int x = (int) xy;  // should be 4
   int y = xy >> 32;  // should be -1
   EM_ASM({
     out('received ' + [$0, $1, $2, $3] + '.');
   }, w, x, y, z);
+  return 42;
 }
 
 int main() {
@@ -23,15 +24,23 @@ int main() {
     // Note that these would need to use BigInts if the file were built with
     // -s WASM_BIGINT
 #if DIRECT
-    dynCall_viji($0, 1, 4, 0xffffffff, 9);
+    console.log('Received ' + dynCall_iiji($0, 1, 4, 0xffffffff, 9));
+    return;
+#endif
+#if DYNAMIC_SIG
+    console.log('Received ' + dynCall('iiji', $0, [1, 4, 0xffffffff, 9]));
     return;
 #endif
 #if EXPORTED
-    Module['dynCall_viji']($0, 1, 4, 0xffffffff, 9);
+    console.log('Received ' + Module['dynCall_iiji']($0, 1, 4, 0xffffffff, 9));
+    return;
+#endif
+#if EXPORTED_DYNAMIC_SIG
+    console.log('Received ' + Module['dynCall']('iiji', $0, [1, 4, 0xffffffff, 9]));
     return;
 #endif
 #if FROM_OUTSIDE
-    eval("Module['dynCall_viji'](" + $0 + ", 1, 4, 0xffffffff, 9)");
+    eval("console.log('Received ' + Module['dynCall_iiji'](" + $0 + ", 1, 4, 0xffffffff, 9))");
     return;
 #endif
     throw "no test mode";
