@@ -25,12 +25,12 @@ int main(int argc, char **argv)
 
   int totalMemory = EM_ASM_INT({ return HEAP8.length });
   char *buf3 = (char*)malloc(totalMemory+1);
-  buf3[argc] = (int)buf2;
-  if (argc % 7 == 6) printf("%d\n", (int)memcpy(buf3, buf1, argc));
+  buf3[argc] = (long)buf2;
+  if (argc % 7 == 6) printf("%ld\n", (long)memcpy(buf3, buf1, argc));
   char *buf4 = (char*)malloc(100);
   float *buf5 = (float*)malloc(100);
   //printf("totalMemory: %d bufs: %d,%d,%d,%d,%d\n", totalMemory, buf1, buf2, buf3, buf4, buf5);
-  assert((int)buf4 > (int)totalMemory && (int)buf5 > (int)totalMemory);
+  assert((long)buf4 > (long)totalMemory && (long)buf5 > (long)totalMemory);
 
   printf("*%s,%.3f*\n", buf1, buf2[0]); // the old heap data should still be there
 
