@@ -3,6 +3,7 @@
 // University of Illinois/NCSA Open Source License.  Both these licenses can be
 // found in the LICENSE file.
 
+#include <stdbool.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <assert.h>
@@ -21,15 +22,16 @@ void looper() {
     double now = emscripten_get_now();
     double msecsPerFrame = (now - frame0) / (numFrames-1); // Sub one to account for intervals vs endpoints
     printf("Avg. msecs/frame: %f\n", msecsPerFrame);
-#ifdef REPORT_RESULT
-    int result = (msecsPerFrame < 5); // Expecting to run extremely fast unthrottled, and certainly not bounded by vsync, so less than common 16.667 msecs per frame (this is assuming 60hz display)
-    REPORT_RESULT(result);
-#endif
     emscripten_cancel_main_loop();
+    // Expecting to run extremely fast unthrottled, and certainly not bounded by
+    // vsync, so less than common 16.667 msecs per frame (this is assuming 60hz
+    // display)
+    exit((msecsPerFrame < 5) ? 0 : 1);
   }
 }
 
 int main() {
   emscripten_set_main_loop(looper, 1, 0);
   emscripten_set_main_loop_timing(EM_TIMING_SETIMMEDIATE, 0);
+  return 99;
 }
