@@ -1,46 +1,45 @@
 // EMSCRIPTEN_START_ASM
-function instantiate(o, p, q) {
+function instantiate(q, r, s) {
  var a = new ArrayBuffer(16);
  var b = new Int32Array(a);
  var c = new Float32Array(a);
- function j(r) {
-  return b[r];
+ function d(e) {
+  return b[e];
  }
- function k(r, s) {
-  b[r] = s;
+ function f(e, g) {
+  b[e] = g;
  }
- function l(s) {
-  c[2] = s;
+ function h(g) {
+  c[2] = g;
  }
- function m() {
+ function i() {
   return c[2];
  }
- function n(t, u, v) {
-  var d = new t.Int8Array(v);
-  var e = u.emscripten_glVertexAttrib4fv;
-  var f = 6191184;
+ function m(n, o, p) {
+  var j = new n.Int8Array(p);
+  var k = o.emscripten_glVertexAttrib4fv;
+  var l = 6191184;
   
 // EMSCRIPTEN_START_FUNCS
-function i() {
- return g(10, 20) + h(30);
+function v() {
+ return t(10, 20) + u(30);
+}
+function t(a, b) {
+ return u(a + b);
 }
 
-
-
-function g(a, b) {
- return h(a + b);
-}
-function h(a) {
+function u(a) {
  return a + 1;
 }
 
+
 // EMSCRIPTEN_END_FUNCS
 
   return {
-   "main": i
+   "main": v
   };
  }
- return n({
+ return m({
   "Int8Array": Int8Array,
   "Int16Array": Int16Array,
   "Int32Array": Int32Array,
@@ -50,12 +49,10 @@ function h(a) {
   "Float32Array": Float32Array,
   "Float64Array": Float64Array,
   "NaN": NaN,
-  "Infinity": Infinity,
+  Infinity: Infinity,
   "Math": Math
- }, o, p.buffer);
+ }, q, r.buffer);
 }
-
-
 // EMSCRIPTEN_END_ASM
 
 // EMSCRIPTEN_GENERATED_FUNCTIONS
