@@ -3,9 +3,9 @@
 // University of Illinois/NCSA Open Source License.  Both these licenses can be
 // found in the LICENSE file.
 
+#include <cassert>
 #include <stdio.h>
 #include <pthread.h>
-#include <emscripten/threading.h>
 
 int result = 0;
 
@@ -27,14 +27,8 @@ int main(void) {
   pthread_t thread;
   puts("a");
   pthread_create(&thread, NULL, thread_func, NULL);
-  if (emscripten_has_threading_support()) {
-    pthread_join(thread, NULL);
-  } else {
-    result = 1;
-  }
+  pthread_join(thread, NULL);
 
-#ifdef REPORT_RESULT
-  REPORT_RESULT(result);
-#endif
+  assert(result == 1);
   return 0;
 }
