@@ -14,7 +14,5 @@ int main()
 	int ENVIRONMENT_IS_WORKER = EM_ASM_INT(return ENVIRONMENT_IS_WORKER);
 	printf("ENVIRONMENT_IS_WORKER: %d\n", ENVIRONMENT_IS_WORKER);
 	assert(ENVIRONMENT_IS_WORKER);
-#ifdef REPORT_RESULT
-	REPORT_RESULT(1);
-#endif
+	return 0;
 }
