@@ -10,8 +10,6 @@
 #ifndef REPORT_RESULT_H_
 #define REPORT_RESULT_H_
 
-#ifdef __EMSCRIPTEN__
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -23,7 +21,8 @@ void _MaybeReportResult(int result, int sync);
 }
 #endif
 
-#if __EMSCRIPTEN_PTHREADS__
+#if defined __EMSCRIPTEN__ && defined __EMSCRIPTEN_PTHREADS__
+  #include <emscripten.h>
   #include <emscripten/threading.h>
   #define REPORT_RESULT(result) emscripten_async_run_in_main_runtime_thread(EM_FUNC_SIG_VII, _ReportResult, (result), 0)
   #define REPORT_RESULT_SYNC(result) emscripten_sync_run_in_main_runtime_thread(EM_FUNC_SIG_VII, _ReportResult, (result), 1)
@@ -36,19 +35,4 @@ void _MaybeReportResult(int result, int sync);
   #define MAYBE_REPORT_RESULT_SYNC(result) _MaybeReportResult((result), 1)
 #endif
 
-#else
-
-#include <stdio.h>
-#include <stdlib.h>
-
-#define REPORT_RESULT(result)       \
-  do {                              \
-    printf("result: %d\n", result); \
-    exit(result);                   \
-  }
-
-#define REPORT_RESULT_SYNC REPORT_RESULT
-
-#endif // __EMSCRIPTEN__
-
 #endif // REPORT_RESULT_H_
