@@ -4,11 +4,12 @@
 // found in the LICENSE file.
 
 #include <assert.h>
+#include <emscripten/console.h>
 #include <stdio.h>
 #include <stdlib.h>
 
-int totalAllocated = 0;
-int totalFreed = 0;
+int totalAllocated;
+int totalFreed;
 
 extern "C"
 {
@@ -20,7 +21,7 @@ void * __attribute__((noinline)) malloc(size_t size)
 {
 	++totalAllocated;
 	void *ptr = emscripten_builtin_malloc(size);
-	printf("Allocated %zu bytes, got %p. %d pointers allocated total.\n", size, ptr, totalAllocated);
+	emscripten_console_logf("Allocated %zu bytes, got %p. %d pointers allocated total.\n", size, ptr, totalAllocated);
 	return ptr;
 }
 
@@ -28,7 +29,7 @@ void __attribute__((noinline)) free(void *ptr)
 {
 	++totalFreed;
 	emscripten_builtin_free(ptr);
-	printf("Freed ptr %p, %d pointers freed total.\n", ptr, totalFreed);
+	emscripten_console_logf("Freed ptr %p, %d pointers freed total.\n", ptr, totalFreed);
 }
 
 }
@@ -39,14 +40,19 @@ void *out __attribute__((used));
 
 int main()
 {
+	// reset these globals here, to ignore any allocations during startup by the
+	// system
+	totalAllocated = 0;
+	totalFreed = 0;
+
 	for(int i = 0; i < 20; ++i)
 	{
 		void *ptr = malloc(1024 * 1024);
 		out = ptr;
 		free(ptr);
 	}
-	printf("totalAllocated: %d\n", totalAllocated);
+	emscripten_console_logf("totalAllocated: %d\n", totalAllocated);
 	assert(totalAllocated == 20);
-	printf("OK.\n");
+	emscripten_console_logf("OK.\n");
 	return 0;
 }
