@@ -167,7 +167,10 @@ def parse_config_file():
   normalize_config_settings()
 
 
-def generate_config(path, first_time=False):
+def generate_config(path):
+  if os.path.exists(path):
+    exit_with_error(f'config file already exists: `{path}`')
+
   # Note: repr is used to ensure the paths are escaped correctly on Windows.
   # The full string is replaced so that the template stays valid Python.
 
@@ -182,19 +185,13 @@ def generate_config(path, first_time=False):
   node = which('node') or which('nodejs') or 'node'
   config_data = config_data.replace('\'{{{ NODE }}}\'', repr(node))
 
-  abspath = os.path.abspath(os.path.expanduser(path))
   # write
+  utils.write_file(path, config_data)
 
-  utils.write_file(abspath, config_data)
+  print('''\
+An Emscripten settings file has been generated at:
 
-  if first_time:
-    print('''
-==============================================================================
-Welcome to Emscripten!
-
-This is the first time any of the Emscripten tools has been run.
-
-A settings file has been copied to %s, at absolute path: %s
+  %s
 
 It contains our best guesses for the important paths, which are:
 
@@ -202,11 +199,8 @@ It contains our best guesses for the important paths, which are:
   NODE_JS         = %s
   EMSCRIPTEN_ROOT = %s
 
-Please edit the file if any of those are incorrect.
-
-This command will now exit. When you are done editing those paths, re-run it.
-==============================================================================
-''' % (path, abspath, llvm_root, node, __rootpath__), file=sys.stderr)
+Please edit the file if any of those are incorrect.\
+''' % (path, llvm_root, node, __rootpath__), file=sys.stderr)
 
 
 # Emscripten configuration is done through the --em-config command line option
@@ -254,20 +248,29 @@ elif os.path.exists(emsdk_embedded_config):
 elif os.path.exists(user_home_config):
   EM_CONFIG = user_home_config
 else:
+  # No config file found.  Set EM_CONFIG to a default value
+  # that will get reported in the error below.
   if root_is_writable():
-    generate_config(embedded_config, first_time=True)
+    EM_CONFIG = embedded_config
   else:
-    generate_config(user_home_config, first_time=True)
-  sys.exit(0)
+    EM_CONFIG = user_home_config
 
 # We used to support inline EM_CONFIG.
 if '\n' in EM_CONFIG:
   exit_with_error('Inline EM_CONFIG data no longer supported.  Please use a config file.')
 
 EM_CONFIG = os.path.expanduser(EM_CONFIG)
-logger.debug('emscripten config is located in ' + EM_CONFIG)
+
+# This command line flag needs to work even in the absence of a config file, so we must process it
+# here at script import time (otherwise the error below will trigger).
+if '--generate-config' in sys.argv:
+  generate_config(EM_CONFIG)
+  sys.exit(0)
+
 if not os.path.exists(EM_CONFIG):
-  exit_with_error('emscripten config file not found: ' + EM_CONFIG)
+  exit_with_error(f'config file not found: {EM_CONFIG}.  Please create one by hand or run `emcc --generate-config`')
+
+logger.debug('emscripten config is located in ' + EM_CONFIG)
 
 # Emscripten compiler spawns other processes, which can reimport shared.py, so
 # make sure that those child processes get the same configuration file by
