@@ -85,9 +85,14 @@ import argparse
 import tempfile
 import subprocess
 
-sys.path.insert(1, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
+__scriptdir__ = os.path.dirname(os.path.abspath(__file__))
+__rootdir__ = os.path.dirname(__scriptdir__)
+sys.path.append(__rootdir__)
 
 from tools import shared
+from tools import system_libs
+from tools import utils
+from tools.settings import settings
 
 QUIET = (__name__ != '__main__')
 DEBUG = False
@@ -139,6 +144,8 @@ def parse_c_output(lines):
 
   for line in lines:
     arg = line[1:].strip()
+    if '::' in arg:
+      arg = arg.split('::', 1)[1]
     if line[0] == 'K':
       # This is a key
       key = arg
@@ -168,7 +175,7 @@ def parse_c_output(lines):
 
 def gen_inspect_code(path, struct, code):
   if path[0][-1] == '#':
-    path[0] = path[0][:-1]
+    path[0] = path[0].rstrip('#')
     prefix = ''
   else:
     prefix = 'struct '
@@ -192,7 +199,7 @@ def gen_inspect_code(path, struct, code):
   c_ascent(code)
 
 
-def inspect_headers(headers, cpp_opts):
+def inspect_headers(headers, cflags):
   code = ['#include <stdio.h>', '#include <stddef.h>']
   for header in headers:
     code.append('#include "' + header['name'] + '"')
@@ -225,36 +232,49 @@ def inspect_headers(headers, cpp_opts):
   code.append('}')
 
   # Write the source code to a temporary file.
-  src_file = tempfile.mkstemp('.c')
+  src_file = tempfile.mkstemp('.c', text=True)
   show('Generating C code... ' + src_file[1])
-  os.write(src_file[0], shared.asbytes('\n'.join(code)))
+  os.write(src_file[0], '\n'.join(code).encode())
 
   js_file = tempfile.mkstemp('.js')
 
+  # Check sanity early on before populating the cache with libcompiler_rt
+  # If we don't do this the parallel build of compiler_rt will run while holding the cache
+  # lock and with EM_EXCLUSIVE_CACHE_ACCESS set causing N processes to race to run sanity checks.
+  # While this is not in itself serious problem it is wasteful and noise on stdout.
+  # For the same reason we run this early in embuilder.py and emcc.py.
+  # TODO(sbc): If we can remove EM_EXCLUSIVE_CACHE_ACCESS then this would not longer be needed.
+  shared.check_sanity()
+
+  compiler_rt = system_libs.Library.get_usable_variations()['libcompiler_rt'].get_path()
+
   # Close all unneeded FDs.
   os.close(src_file[0])
   os.close(js_file[0])
 
-  # Remove dangerous env modifications
-  env = os.environ.copy()
-  env['EMCC_FORCE_STDLIBS'] = 'libcompiler_rt'
-  env['EMCC_ONLY_FORCED_STDLIBS'] = '1'
-
   info = []
   # Compile the program.
   show('Compiling generated code...')
+
+  if any('libcxxabi' in f for f in cflags):
+    compiler = shared.EMXX
+  else:
+    compiler = shared.EMCC
+
   # -Oz optimizes enough to avoid warnings on code size/num locals
-  cmd = [shared.EMCC] + cpp_opts + ['-o', js_file[1], src_file[1],
-                                    '-O0',
-                                    '-Werror',
-                                    '-Wno-format',
-                                    '-I', shared.path_from_root(),
-                                    '-s', 'BOOTSTRAPPING_STRUCT_INFO=1',
-                                    '-s', 'WARN_ON_UNDEFINED_SYMBOLS=0',
-                                    '-s', 'STRICT=1',
-                                    # Use SINGLE_FILE=1 so there is only a single
-                                    # file to cleanup.
-                                    '-s', 'SINGLE_FILE=1']
+  cmd = [compiler] + cflags + ['-o', js_file[1], src_file[1],
+                               '-O0',
+                               '-Werror',
+                               '-Wno-format',
+                               '-nostdlib',
+                               compiler_rt,
+                               '-sMEMORY64=' + str(settings.MEMORY64),
+                               '-sBOOTSTRAPPING_STRUCT_INFO=1',
+                               '-sLLD_REPORT_UNDEFINED=1',
+                               '-sSTRICT',
+                               # Use SINGLE_FILE=1 so there is only a single
+                               # file to cleanup.
+                               '-sSINGLE_FILE']
 
   # Default behavior for emcc is to warn for binaryen version check mismatches
   # so we should try to match that behavior.
@@ -263,25 +283,29 @@ def inspect_headers(headers, cpp_opts):
   # TODO(sbc): Remove this one we remove the test_em_config_env_var test
   cmd += ['-Wno-deprecated']
 
-  if shared.Settings.LTO:
-    cmd += ['-flto=' + shared.Settings.LTO]
+  if settings.LTO:
+    cmd += ['-flto=' + settings.LTO]
 
   show(shared.shlex_join(cmd))
   try:
-    subprocess.check_call(cmd, env=env)
+    subprocess.check_call(cmd, env=system_libs.clean_env())
   except subprocess.CalledProcessError as e:
     sys.stderr.write('FAIL: Compilation failed!: %s\n' % e.cmd)
     sys.exit(1)
 
   # Run the compiled program.
   show('Calling generated program... ' + js_file[1])
-  info = shared.run_js_tool(js_file[1], stdout=shared.PIPE).splitlines()
+  args = []
+  if settings.MEMORY64:
+    args += ['--experimental-wasm-bigint']
+  info = shared.run_js_tool(js_file[1], node_args=args, stdout=shared.PIPE).splitlines()
 
-  # Remove all temporary files.
-  os.unlink(src_file[1])
+  if not DEBUG:
+    # Remove all temporary files.
+    os.unlink(src_file[1])
 
-  if os.path.exists(js_file[1]):
-    os.unlink(js_file[1])
+    if os.path.exists(js_file[1]):
+      os.unlink(js_file[1])
 
   # Parse the output of the program into a dict.
   return parse_c_output(info)
@@ -299,13 +323,13 @@ def merge_info(target, src):
     target['structs'][key] = value
 
 
-def inspect_code(headers, cpp_opts):
+def inspect_code(headers, cflags):
   if not DEBUG:
-    info = inspect_headers(headers, cpp_opts)
+    info = inspect_headers(headers, cflags)
   else:
     info = {'defines': {}, 'structs': {}}
     for header in headers:
-      merge_info(info, inspect_headers([header], cpp_opts))
+      merge_info(info, inspect_headers([header], cflags))
   return info
 
 
@@ -362,8 +386,9 @@ def main(args):
   global QUIET
 
   default_json_files = [
-      shared.path_from_root('src', 'struct_info.json'),
-      shared.path_from_root('src', 'struct_info_internal.json')
+      utils.path_from_root('src/struct_info.json'),
+      utils.path_from_root('src/struct_info_internal.json'),
+      utils.path_from_root('src/struct_info_cxx.json'),
   ]
   parser = argparse.ArgumentParser(description='Generate JSON infos for structs.')
   parser.add_argument('json', nargs='*',
@@ -379,22 +404,39 @@ def main(args):
                       help='Pass a define to the preprocessor')
   parser.add_argument('-U', dest='undefines', metavar='undefine', action='append', default=[],
                       help='Pass an undefine to the preprocessor')
+  parser.add_argument('--wasm64', action='store_true',
+                      help='use wasm64 architecture')
   args = parser.parse_args(args)
 
   QUIET = args.quiet
 
   # Avoid parsing problems due to gcc specifc syntax.
-  cpp_opts = ['-D_GNU_SOURCE']
+  cflags = ['-D_GNU_SOURCE']
+
+  if args.wasm64:
+    settings.MEMORY64 = 2
 
   # Add the user options to the list as well.
   for path in args.includes:
-    cpp_opts.append('-I' + path)
+    cflags.append('-I' + path)
 
   for arg in args.defines:
-    cpp_opts.append('-D' + arg)
+    cflags.append('-D' + arg)
 
   for arg in args.undefines:
-    cpp_opts.append('-U' + arg)
+    cflags.append('-U' + arg)
+
+  internal_cflags = [
+    '-I' + utils.path_from_root('system/lib/libc/musl/src/internal'),
+    '-I' + utils.path_from_root('system/lib/libc/musl/src/include'),
+    '-I' + utils.path_from_root('system/lib/pthread/'),
+    '-I' + utils.path_from_root('system/lib/wasmfs/'),
+  ]
+
+  cxxflags = [
+    '-I' + utils.path_from_root('system/lib/libcxxabi/src'),
+    '-D__USING_EMSCRIPTEN_EXCEPTIONS__',
+  ]
 
   # Look for structs in all passed headers.
   info = {'defines': {}, 'structs': {}}
@@ -403,7 +445,13 @@ def main(args):
     # This is a JSON file, parse it.
     header_files = parse_json(f)
     # Inspect all collected structs.
-    info_fragment = inspect_code(header_files, cpp_opts)
+    if 'internal' in f:
+      use_cflags = cflags + internal_cflags
+    elif 'cxx' in f:
+      use_cflags = cflags + cxxflags
+    else:
+      use_cflags = cflags
+    info_fragment = inspect_code(header_files, use_cflags)
     merge_info(info, info_fragment)
 
   output_json(info, args.output)
