@@ -29,9 +29,8 @@ def get(ports, settings, shared):
 
     # build
     srcs = [
-      'blocksort.c', 'bzip2recover.c', 'compress.c', 'decompress.c',
-      'huffman.c', 'randtable.c', 'unzcrash.c', 'bzip2.c', 'bzlib.c',
-      'crctable.c', 'dlltest.c', 'mk251.c', 'spewG.c',
+      'blocksort.c', 'compress.c', 'decompress.c', 'huffman.c',
+      'randtable.c', 'bzlib.c', 'crctable.c',
     ]
     commands = []
     o_s = []
@@ -49,7 +48,7 @@ def get(ports, settings, shared):
 
 
 def clear(ports, settings, shared):
-  shared.Cache.erase_file('libbz2.a')
+  shared.Cache.erase_lib('libbz2.a')
 
 
 def process_args(ports):
