@@ -6,6 +6,7 @@
 import logging
 import os
 import shutil
+from pathlib import Path
 
 TAG = 'version_1'
 HASH = '929e8d6003c06ae09593021b83323c8f1f54532b67b8ba189f4aedce52c25dc182bac474de5392c46ad5b0dea5a24928e4ede1492d52f4dd5cd58eea9be4dba7'
@@ -28,7 +29,7 @@ def get(ports, settings, shared):
     shutil.rmtree(dest_path, ignore_errors=True)
     shutil.copytree(source_path, dest_path)
 
-    open(os.path.join(dest_path, 'include', 'ogg', 'config_types.h'), 'w').write(config_types_h)
+    Path(dest_path, 'include', 'ogg', 'config_types.h').write_text(config_types_h)
 
     header_dir = os.path.join(ports.get_include_dir(), 'ogg')
     shutil.rmtree(header_dir, ignore_errors=True)
@@ -40,7 +41,7 @@ def get(ports, settings, shared):
 
 
 def clear(ports, settings, shared):
-  shared.Cache.erase_file('libogg.a')
+  shared.Cache.erase_lib('libogg.a')
 
 
 def process_args(ports):
