@@ -188,12 +188,7 @@ if EMPROFILE == 1:
 
     @staticmethod
     def record_subprocess_spawn(process_pid, process_cmdline):
-      expanded_cmdline = []
-      for item in process_cmdline:
-        if item.startswith('@'):
-          expanded_cmdline += response_file.read_response_file(item)
-        else:
-          expanded_cmdline.append(item)
+      expanded_cmdline = response_file.substitute_response_files(process_cmdline)
 
       with ToolchainProfiler.log_access() as f:
         f.write(',\n{"pid":' + ToolchainProfiler.mypid_str + ',"subprocessPid":' + str(os.getpid()) + ',"op":"spawn","targetPid":' + str(process_pid) + ',"time":' + ToolchainProfiler.timestamp() + ',"cmdLine":["' + '","'.join(ToolchainProfiler.escape_args(expanded_cmdline)) + '"]}')
